# Copyright 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import api, fields, models


class AdefpatProjectReport(models.TransientModel):
    _name = "adefpat.project.report.wizard"
    _description = "Wizard rapports projets"

    period_start = fields.Date("Début de période", required=True)
    period_end = fields.Date(
        "Fin de période", required=True, default=fields.Date.today()
    )
    export_type = fields.Selection(
        [
            ("activity", "Activité"),
            ("benef", "Bénéficiaires"),
            ("cf", "Consultants Formateurs"),
            ("results", "Résultats"),
        ],
        string="Type de rapport",
        required=True,
        default="activity",
    )
    is_fd = fields.Boolean("Convention FD uniquement", default=True)
    is_closed = fields.Boolean("Projet clôturé sur la période uniquement", default=True)

    @api.onchange("export_type")
    def onchange_bools(self):
        # Default values depending of export_type
        if self.export_type in ["benef", "cf"]:
            self.is_closed = False
        else:
            self.is_closed = True

    # function will trigger from the show report button
    @api.multi
    def show_report(self):
        # Get Projects with at least 1 task on period
        project_ids = self.env["project.task"].search([
            ("date_deadline", "<=", self.period_end),
            ("date_deadline", ">=", self.period_start),
        ]).mapped("project_id")

        if self.is_fd:
            project_ids = project_ids.filtered(lambda p: p.type_convention_id.is_fd)
        if self.is_closed:
            project_ids = project_ids.filtered(
                lambda p: p.closing_date and p.closing_date <= self.period_end
                and p.closing_date >= self.period_start)

        # Select view
        res_model = "project.project"
        domain = [("id", "in", project_ids.ids)]
        limit = len(project_ids)
        # Rapport Activité
        if self.export_type == "activity":
            tree_view = self.env.ref(
                "adefpat_project.view_adefpat_project_reporting_tree"
            )
        # Rapport Porteurs de projets
        elif self.export_type == "benef":
            tree_view = self.env.ref(
                "adefpat_project.view_adefpat_participant_reporting_tree"
            )
            res_model = "res.partner.porteur.project"
            domain = [("project_id", "in", project_ids.ids)]
            limit = self.env["res.partner.porteur.project"].search_count(domain)
        # Rapport Formateurs
        elif self.export_type == "cf":
            tree_view = self.env.ref("adefpat_project.view_adefpat_formateur_reporting_tree")
            res_model = "adefpat.formateur.project.report"
            domain = [("project_id", "in", project_ids.ids)]
            limit = self.env["adefpat.formateur.project.report"].search_count(domain)
        elif self.export_type == "results":
            tree_view = self.env.ref("adefpat_project.view_adefpat_project_result_tree")

        action_name = "Rapport %s | Du %s au %s" % (
            dict(self._fields["export_type"].selection).get(self.export_type),
            fields.Date.to_string(self.period_start),
            fields.Date.to_string(self.period_end),
        )

        return {
            "type": "ir.actions.act_window",
            "name": action_name,
            "res_model": res_model,
            "view_mode": "tree",
            "views": [(tree_view.id, "tree")],
            "target": "main",
            "domain": domain,
            "context": {
                "period_start": self.period_start,
                "period_end": self.period_end,
                "is_fd": self.is_fd,
                "is_closed": self.is_closed
            },
            "limit": limit,
        }
